
;    Copyright (c) 2007 Richard Ibbotson <richard.ibbotson@btinternet.com>
;
;    Permission is hereby granted, free of charge, to any person
;    obtaining a copy of this software and associated documentation
;    files (the "Software"), to deal in the Software without
;    restriction, including without limitation the rights to use, copy,
;    modify, merge, publish, distribute, sublicense, and/or sell copies
;    of the Software, and to permit persons to whom the Software is
;    furnished to do so, subject to the following conditions:
;
;    The above copyright notice and this permission notice shall be
;    included in all copies or substantial portions of the Software.
;
;    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
;    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
;    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
;    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
;    DEALINGS IN THE SOFTWARE.
;



;**********************************************************************
;                                                                     *
;    Filename:	    PS2.asm                                           *
;    Date:             2nd October 2007                               *
;    File Version:      V 0.1                                         *
;                                                                     *
;    Author:      Richard Ibbotson                                    *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                    16F688.lkr                                       *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
; 
;
; 16F688 Chip Connections
;
; PORT		PIN		CONFIGURATION
; ==================================
; A0        13      PGM Header
; A1        12      PGM Header
; A2        11      PS2 ACK + 4.7K pullup to Vcc (not used in this code)
; C0        10      PS2 DAT + 4.7K pullup to Vcc
; C1        9       PS2 CMD 
; C2        8       PS2 CLK
; C3        7       PS2 ATT
; C4        6       Serial TX
; C5        5       Serial RX
; MCLR      4       PGM Header
; OSC2      3       N/C
; OSC1      2       N/C
; GND       14		GND
; VCC       1		Vcc
;

	list      p=16F688        ; list directive to define processor
	#include <p16F688.inc>    ; 
	errorlevel  -302          ; suppress message 302 from list file

; Here include Configuration settings
	__CONFIG   _CP_OFF & _CPD_OFF & _BOD_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _FCMEN_OFF & _IESO_OFF 

; OSC = INTOSCIO
; Watchdog = Disabled
; Power up Timer  = Enabled
; MCLR = Reset
; Brownout = Enabled, SBODEN disabled
; Switchover = Disabled
; Failsave = Disabled
; Code Protect Disable


;	Data Register Variables 	
	CBLOCK 0x20
; PS2 output buffer
	outlength		; number of bytes to send
	outbuf1			; start byte = 01			
	outbuf2	
	outbuf3
	outbuf4	
	outbuf5
	outbuf6	
	outbuf7
	outbuf8	
	outbuf9
	outbuf10
	outbuf11
	outbuf12	
	outbuf13
	outbuf14	
	outbuf15
	outbuf16	
	outbuf17
	outbuf18
	outbuf19	
	outbuf20
	outbuf21	
	outbuf22

; PS2 input buffer
	inbuf1			;Null					
	inbuf2			; Mode Number	
	inbuf3			; Mode changed
	inbuf4			; 'A' dpad switches
	inbuf5			; 'B' buttons
	inbuf6			; 'C' joystick right X
	inbuf7			; 'D' joystick right Y
	inbuf8			; 'E' joystick left X
	inbuf9			; 'F' joystick left Y
	inbuf10			; 'G' dpad right
	inbuf11			; 'H' dpad left
	inbuf12			; 'I' dpad up
	inbuf13			; 'J' dpad down
	inbuf14			; 'K' triangle
	inbuf15			; 'L' circle
	inbuf16			; 'M' cross
	inbuf17			; 'N' sqaure
	inbuf18			; 'O' Left 1
	inbuf19			; 'P' Right 1
	inbuf20			; 'Q' Left 2
	inbuf21			; 'R' right 2
	inbuf22			; 'S'

	rxchar		; buffer for received serial character
	txchar		; buffer for transmit serial char

	outcount		; counter for the table read
	outshift		; output shift register
	PS2bytecount	; byte counter for PS2 transfer
	PS2bitcount		; bit counter for PS2 transfer
	delaycount		; counter for dalay loop
	stickval		; stick value to be mapped
	delayvar		; used for msdelay
	pausevar		; used for long pause
	timerhi			; hi byte of keep alive timer
	timerlo			; low byte of keep alive timer
	ENDC

	#define		BANK0	bcf		STATUS,RP0
	#define		BANK1	bsf		STATUS,RP0

;-----------------------------------------------------------------

	ORG 0x000				; Power on Reset
	nop
	clrf PCLATH
	goto start


	ORG 0x004				; Interrupt handler goes here

; Interrupts are not used
 
 	return					; return



; ---------- Main Program Starts Here ----------------------------

start
	
	call	chipinit		; Initialize pins, oscillator, etc

	call 	uartinit		; Set up the EUSART for 19200


	BANK0


waitrxchar	
	clrf	timerhi
	clrf	timerlo
waitrxloop
	; wait to receive a single serial character from the C3024 at 19200
	btfsc	PIR1,RCIF		; get rx ready flag
	goto	getrxchar
	decf	timerlo,f
	btfss	STATUS,Z
	goto	waitrxloop
	decf	timerhi,f
	btfss	STATUS,Z
	goto	waitrxloop
	goto	keepalive
getrxchar
	movf	RCREG, w		; get received char
	movwf	rxchar			; put in the rxchar register

; check the character to determine the action

	movlw	'X'				; is it an 'X' Read encoded Buttons
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandX

	
	movlw	'A'				; is it an 'A' Read button set 1
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandA
	
	movlw	'B'				; is it a 'B' Read button set 2
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandB	
	
	movlw	'C'				; is it a 'C' Read stick right X
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandC
	
	movlw	'D'				; is it a 'D' Read stick right Y
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandD
	
	movlw	'E'				; is it an 'E' Read stick left X
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandE
	
	movlw	'F'				; is it an 'F' Read stick left Y
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandF
	movlw	'c'				; is it a 'c' Read stick right X
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandc
	
	movlw	'd'				; is it a 'd' Read stick right Y
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandd
	
	movlw	'e'				; is it an 'e' Read stick left X
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commande
	
	movlw	'f'				; is it an 'f' Read stick left Y
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandf
	movlw	'G' - 1			; is it >=  to 'G'
	subwf	rxchar, w
	btfss	STATUS,C
	goto	decodecmd1
	movlw	'R'				; is it <= to 'R'
	subwf	rxchar,w
	btfss	STATUS,C
	goto	commandGR


decodecmd1
	movlw	'W'				; is it a 'W' lock DS2 mode
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandW
	movlw	'Y'				; is it a 'Y' lock analog mode
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandY

	movlw	'Z'				; is it a 'Z' lock digital mode
	subwf	rxchar, w
	btfsc	STATUS,Z
	goto	commandZ


	movf	rxchar, w
	
	movwf	txchar

waittxready
	btfss	PIR1, TXIF		; get tx ready flag
	goto 	waittxready
	movf	txchar, w
	movwf	TXREG			; echo the char


 
	goto 	waitrxchar			; Repeat forever.

 
; ----------- Routines for Starting the Chip ---------

chipinit
	BANK0					; Memory bank 0
	movlw	0eh				; set output bits high
	movwf	PORTC
	movlw	07h				;
	movwf	CMCON0			; Comparator mode off

	BANK1					; Switch to memory bank 1
	movlw	21h				; Set the directions for port C
	movwf	TRISC			; 0,5 are in, 1,2,3,4 are out

	movlw	3ch				; Set the directions for port A
	movwf	TRISA			; 0,1 are out, 2,3,4,5 are in

	clrf	ANSEL			; Turn off A/D Converters

	bsf	OSCCON,6			; Set these three
	bsf	OSCCON,5			; bits to enable the
	bsf	OSCCON,4			; 8 MHz internal oscillator
	
	BANK0
	return

uartinit
	BANK0
	
	movlw 	B'00100100'		; Serial Port Transmit enable Async High baud
	movwf	TXSTA	

	movlw 	B'10010000'		; Serial Port Receive enabled
	movwf	RCSTA	
		
	
	movlw	D'103'			; for baud rate of 19200
	movwf	SPBRG			
	clrf	SPBRGH			; 

	bsf		BAUDCTL,BRG16	;

	return			




; ----------------- Other Subroutines Go Here

;------------------------------------------------------------------------------
; get the first buttons field
; buttons are inverse
commandA					; routine to process 'A' command
	call	readshort		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf4, w		; get the button values
	movwf	txchar
	goto	waittxready		; send the reply

;------------------------------------------------------------------------------
; get the second buttons field
; buttons are inverse
commandB					; routine to process 'B' command
	call	readshort		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf5, w		; get the switch values
	movwf	txchar
	goto	waittxready		; send the reply

;------------------------------------------------------------------------------
; get both buttons fields and encode value
; buttons are inverse
commandX					; routine to process 'X' command this is a encode of the returned buttons
	call	readshort		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	call	encodebutton	; priority encode the buttons in inbuf4 and 5 to w
							; 0 = no button pressed
	movwf	txchar
	goto	waittxready		; send the reply

;------------------------------------------------------------------------------
; get the joystick values and map to range 10 to 190

commandC					; routine to process 'C' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf6, w		; get the right X value
	movwf	stickval
	call	mapstick
	movwf	txchar
	goto	waittxready		; send the reply

commandD					; routine to process 'D' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf7, w		; get the right Y value
	movwf	stickval
	call	mapstick
	movwf	txchar
	goto	waittxready		; send the reply

commandE					; routine to process 'E' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf8, w		; get the left X value
	movwf	stickval
	call	mapstick
	movwf	txchar
	goto	waittxready		; send the reply

commandF					; routine to process 'F' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf9, w		; get the left Y value
	movwf	stickval
	call	mapstick
	movwf	txchar
	goto	waittxready		; send the reply
;------------------------------------------------------------------------------
; get the joystick values and map to range 33 to 55

commandc					; routine to process 'c' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf6, w		; get the right X value
	movwf	stickval
	call	gyrostick
	movwf	txchar
	goto	waittxready		; send the reply

commandd					; routine to process 'd' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf7, w		; get the right Y value
	movwf	stickval
	call	gyrostick
	movwf	txchar
	goto	waittxready		; send the reply

commande					; routine to process 'e' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf8, w		; get the left X value
	movwf	stickval
	call	gyrostick
	movwf	txchar
	goto	waittxready		; send the reply

commandf					; routine to process 'f' command
	call	readdata		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movf	inbuf9, w		; get the left Y value
	movwf	stickval
	call	gyrostick
	movwf	txchar
	goto	waittxready		; send the reply
;------------------------------------------------------------------------------
; Read the button pressure values and map to range 0 to 15

commandGR					; routine to process 'G' to 'R' commands
	call	readlong		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	movlw	inbuf10 - 'G' 	; make the offset to the inbuf
	addwf	rxchar,w
	movwf	FSR
	movf	INDF, w			; get the pressure value
	movwf	txchar
	rrf		txchar,f		; shift to fet value  0 to 15
	rrf		txchar,f
	rrf		txchar,f
	rrf		txchar,f
	movlw	0fh
	andwf	txchar,f
	goto	waittxready		; send the reply

;---------------------------------------------------------
; set the controller to DS2 mode and lock
commandW					; routine to process 'W' command
	call	configenter
	call	PS2inout
	call	msdelay
	call	analoglock		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	call	msdelay
	call	DS2enter
	call	PS2inout
	call	DS2exit
	call	PS2inout
	call	configexit
	call	PS2inout
	call	readdata
	call	msdelay
	call	PS2inout
	movf	inbuf2, w		; get the mode value
	movwf	txchar
	goto	waittxready		; send the reply


;---------------------------------------------------------
; set the controller to analog mode and lock
commandY					; routine to process 'Y' command
	call	configenter
	call	PS2inout
	call	msdelay
	call	analoglock		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	call	msdelay
	call	configexit
	call	PS2inout
	call	msdelay
	call	readdata
	call	PS2inout
	movf	inbuf2, w		; get the mode value
	movwf	txchar
	goto	waittxready		; send the reply

;---------------------------------------------------------
; set the controller to digital mode and lock

commandZ					; routine to process 'Z' command
	call	configenter
	call	PS2inout
	call	msdelay
	call	digitallock		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	call	msdelay
	call	configexit
	call	PS2inout
	call	msdelay
	call	readdata
	call	PS2inout
	movf	inbuf2, w		; get the mode value
	movwf	txchar
	goto	waittxready		; send the reply

;------------------------------------------------------------------------------
; read contolller to keep the mode set
; 
keepalive					; routine to keep alive
	call	readshort		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	goto	waitrxchar		; don't reply

;----------------------------------------------------------------------------------------
;


readdata
	movlw	HIGH tblreaddata	;setup high table address
	movwf	PCLATH
	clrw
	movwf	outcount		; put it into table read counter
	call	tblreaddata		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
readloop
	movlw	HIGH tblreaddata	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblreaddata		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	readloop
;---------------------------------------------------------------------------------------
;
readshort
	movlw	HIGH tblreadshort	;setup high table address
	movwf	PCLATH
	clrw
	movwf	outcount		; put it into table read counter
	call	tblreadshort		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
readshortloop
	movlw	HIGH tblreadshort	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblreadshort		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	readshortloop
;---------------------------------------------------------------------------------------
;
readlong
	movlw	HIGH tblreadlong	;setup high table address
	movwf	PCLATH
	clrw
	movwf	outcount		; put it into table read counter
	call	tblreadlong		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
readlongloop
	movlw	HIGH tblreadlong	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblreadlong		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	readlongloop
;--------------------------------------------------------------------------------------
;
configenter
	movlw	HIGH tblconfigenter	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblconfigenter		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
centerloop
	movlw	HIGH tblconfigenter	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblconfigenter		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	centerloop
;---------------------------------------------------------------------------------------
;	
configexit
	movlw	HIGH tblconfigexit	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblconfigexit		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
cexitloop
	movlw	HIGH tblconfigexit	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblconfigexit		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	cexitloop
;--------------------------------------------------------------------------------------
;
DS2enter
	movlw	HIGH tblDS2native	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblDS2native		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
DS2entloop
	movlw	HIGH tblDS2native	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblDS2native		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	DS2entloop
;---------------------------------------------------------------------------------------
;
DS2exit
	movlw	HIGH tblDS2exit	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblDS2exit		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
DS2exitloop
	movlw	HIGH tblDS2exit	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblDS2exit		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	DS2exitloop	
;---------------------------------------------------------------------------------------
;
viben
	movlw	HIGH tblvibena	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblvibena		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
vibenloop
	movlw	HIGH tblvibena	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblvibena		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	vibenloop	
;---------------------------------------------------------------------------------------
;
vibdis
	movlw	HIGH tblvibdis	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblvibdis		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
vibdisloop
	movlw	HIGH tblvibdis	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblvibdis		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	vibdisloop	
;---------------------------------------------------------------------------------------
;
analoglock
	movlw	HIGH tblanalock	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblanalock		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
analogloop
	movlw	HIGH tblanalock	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblanalock		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	analogloop
;-------------------------------------------------------------------------------------
;

digitallock
	movlw	HIGH tbldiglock	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tbldiglock		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
digitalloop
	movlw	HIGH tbldiglock	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tbldiglock		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	digitalloop

;-------------------------------------------------------------------------------------

; this is done this away to enable flexibility in assignment of codes to buttons
; change the order to change priority
; change the mapping if required
encodebutton
	btfss	inbuf4,7		
	retlw	1h
	btfss	inbuf4,6
	retlw	2h
	btfss	inbuf4,5		
	retlw	3h
	btfss	inbuf4,4
	retlw	4h
	btfss	inbuf4,3		
	retlw	5h
	btfss	inbuf4,2
	retlw	6h
	btfss	inbuf4,1		
	retlw	7h
	btfss	inbuf4,0
	retlw	8h
	btfss	inbuf5,7		
	retlw	9h
	btfss	inbuf5,6
	retlw	0ah
	btfss	inbuf5,5		
	retlw	0bh
	btfss	inbuf5,4
	retlw	0ch
	btfss	inbuf5,3		
	retlw	0dh
	btfss	inbuf5,2
	retlw	0eh
	btfss	inbuf5,1		
	retlw	0fh
	btfss	inbuf5,0
	retlw	10h
	retlw	00h
; end of encode

;-----------------------------------------------------------------------------------
;Joystick mapping
; This maps the value retuned from the stick (0 to FF) into
; values which are better for RoboBasic
mapstick
	
	movlw	HIGH tblmapping	;setup high table address
	movwf	PCLATH
	movf	stickval,w
	movwf	PCL

gyrostick
	
	movlw	HIGH gyromapping	;setup high table address
	movwf	PCLATH
	movf	stickval,w
	movwf	PCL

;----------------------------------------------------------------------------------
;Delay and pause

msdelay
	clrf	delayvar	; count for delay
delayloop
	nop
	nop
	nop
	nop
	nop
	decf	delayvar,f
	btfss	STATUS,Z
	goto delayloop
	return

pause
	clrf	pausevar	; count for pause
pauseloop
	call	msdelay
	decf	delayvar,f
	btfss	STATUS,Z
	goto pauseloop
	return
;-----------------------------------------------------------------------------------
; here we go to make the PS2 input/output
	
PS2inout
	clrf	PS2bytecount
	bcf		PORTC,3			; set ATT low
	
PS2loopbyte
	movlw	outbuf1			; fsr points to output buffer	
	movwf	FSR				
	movf	PS2bytecount, w ; get the byte counter
	addwf	FSR	,f			; add the offset
	movf	INDF,w			; get the value
	movwf	outshift		; put it into output shift reg

	movlw	inbuf1			; fsr points to input buffer	
	movwf	FSR				
	movf	PS2bytecount, w ; get the byte counter
	addwf	FSR, f				; add the offset
	movf	INDF,w			; get the value	


	movlw	08h				; eight bits
	movwf	PS2bitcount		; ready for first bit

PS2loopbit
	rrf		outshift,f		; shift the output
	btfsc	STATUS, C		; is carry set
	goto 	PS2outhi

	bcf		PORTC, 1		; set cmd bit low
	bcf		PORTC, 2		; set clk bit low
	goto	waitlow
PS2outhi
	bsf		PORTC, 1		; set cmd bit high
	bcf		PORTC, 2		; set clk bit low

waitlow
	rrf		INDF, f
	bsf		PORTC,2			; set clock high
	btfsc	PORTC,0			; get the data
	goto 	sethigh
	bcf		INDF, 7			; set the highest bit
	goto	indone
sethigh
	bsf		INDF, 7          ; clear the highest bit
indone

	decf	PS2bitcount,f
	btfss	STATUS, Z
	goto	PS2loopbit
	bsf		PORTC, 1		; remove data
	incf	PS2bytecount, f	; next byte count
	movf	outlength, w
	subwf	PS2bytecount,w
	btfss	STATUS, Z
	
	goto	PS2loopbyte
	bsf		PORTC, 3		; remove ATT
	return	
;-------------------------------------------------------------------------
;

	
; place the tables in seperate segment
	ORG 0x0500	

;-------------------------------------------------------------------------
;These are the PS2 command strings

tblreaddata
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 42h, 00h, 01h, 80h, 00h, 00h, 00h, 00h 

tblconfigenter
	addwf	PCL, f			; add the table offset
	dt		05h, 01h , 43h, 00h, 01h, 00h

tblconfigexit
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 43h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

tbldiglock
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 44h, 00h, 00h, 03h, 00h, 00h, 00h, 00h

tblanalock
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 44h, 00h, 01h, 03h, 00h, 00h, 00h, 00h 

tblvibena
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 4dh, 00h, 00h, 01h, 0ffh, 0ffh, 0ffh, 0ffh 

tblvibdis
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 4dh, 00h, 0ffh, 0ffh, 0ffh, 0ffh, 0ffh, 0ffh

tblDS2native
 	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 4fh, 00h, 0ffh, 0ffh, 03h, 00h, 00h, 00h

tblDS2exit

	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 43h, 00h, 00h, 5ah, 5ah, 5ah,5ah, 5ah

tblquerymode
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 45h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

tblqueryana
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 41h, 00h, 5ah, 5ah, 5ah, 5ah, 5ah, 5ah

tblreadshort
	addwf	PCL, f			; add the table offset
	dt		05h, 01h , 42h, 00h, 01h, 80h 

tblreadlong

	addwf	PCL, f			; add the table offset
	dt		16h, 01h , 42h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h  

;----------------------------------------------------------------------------------------------------	
; This table is used to map the joystick output valuse  ( 0 to 0ffH) to a range of values useful
; to Robobasic. In this case in the range 33 to 55, but this can be changed
	ORG	0x600
gyromapping

	dt		21h, 21h, 21h, 21h, 21h, 21h, 22h, 22h, 22h, 22h, 22h, 22h, 22h, 22h, 22h, 22h
	dt		22h, 22h, 23h, 23h, 23h, 23h, 23h, 23h, 23h, 23h, 23h, 23h, 23h, 23h, 24h, 24h
	dt		24h, 24h, 24h, 24h, 24h, 24h, 24h, 24h, 25h, 25h, 25h, 25h, 25h, 25h, 25h, 25h
	dt		25h, 25h, 25h, 25h, 26h, 26h, 26h, 26h, 26h, 26h, 26h, 26h, 26h, 26h, 26h, 26h
	dt		27h, 27h, 27h, 27h, 27h, 27h, 27h, 27h, 27h, 27h, 27h, 28h, 28h, 28h, 28h, 28h
	dt		28h, 28h, 28h, 28h, 28h, 28h, 28h, 28h, 29h, 29h, 29h, 29h, 29h, 29h, 29h, 29h
	dt		29h, 29h, 29h, 29h, 2ah, 2ah, 2ah, 2ah, 2ah, 2ah, 2ah, 2ah, 2ah, 2ah, 2ah, 2bh
	dt		2bh, 2bh, 2bh, 2bh, 2bh, 2bh, 2bh, 2bh, 2bh, 2bh, 2ch, 2ch, 2ch, 2ch, 2ch, 2ch
	dt		2ch, 2ch, 2ch, 2ch, 2ch, 2ch, 2dh, 2dh, 2dh, 2dh, 2dh, 2dh, 2dh, 2dh, 2dh, 2dh
	dt		2dh, 2dh, 2eh, 2eh, 2eh, 2eh, 2eh, 2eh, 2eh, 2eh, 2eh, 2eh, 2eh, 2eh, 2fh, 2fh
	dt		2fh, 2fh, 2fh, 2fh, 2fh, 2fh, 2fh, 2fh, 30h, 30h, 30h, 30h, 30h, 30h, 30h, 30h
	dt		30h, 30h, 30h, 30h, 30h, 31h, 31h, 31h, 31h, 31h, 31h, 31h, 31h, 31h, 31h, 31h
	dt		32h, 32h, 32h, 32h, 32h, 32h, 32h, 32h, 32h, 32h, 32h, 32h, 33h, 33h, 33h, 33h
	dt		33h, 33h, 33h, 33h, 33h, 33h, 33h, 33h, 34h, 34h, 34h, 34h, 34h, 34h, 34h, 34h
	dt		34h, 34h, 34h, 35h, 35h, 35h, 35h, 35h, 35h, 35h, 35h, 35h, 35h, 35h, 35h, 36h
	dt		36h, 36h, 36h, 36h, 36h, 36h, 36h, 36h, 36h, 36h, 37h, 37h, 37h, 37h, 37h, 37h



;------------------------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------------------	
; This table is used to map the joystick output valuse  ( 0 to 0ffH) to a range of values useful
; to Robobasic. In this case in the range 10 to 190, but this can be changed
	ORG	0x700
tblmapping

	dt		0ah, 0ah, 0bh, 0ch, 0ch, 0dh, 0eh, 0eh, 0fh, 10h, 11h, 11h, 12h, 13h, 13h, 14h
	dt		15h, 16h, 16h, 17h, 18h, 18h, 19h, 1ah, 1ah, 1bh, 1ch, 1dh, 1dh, 1eh, 1fh, 1fh
	dt		20h, 21h, 22h, 22h, 23h, 24h, 24h, 25h, 26h, 26h, 27h, 28h, 29h, 29h, 2ah, 2bh
	dt		2bh, 2ch, 2dh, 2eh, 2eh, 2fh, 30h, 30h, 31h, 32h, 32h, 33h, 34h, 35h, 35h, 36h
	dt		37h, 37h, 38h, 39h, 3ah, 3ah, 3bh, 3ch, 3ch, 3dh, 3eh, 3eh, 3fh, 40h, 41h, 41h
	dt		42h, 43h, 43h, 44h, 45h, 46h, 46h, 47h, 48h, 48h, 49h, 4ah, 4ah, 4bh, 4ch, 4dh
	dt		4dh, 4eh, 4fh, 4fh, 50h, 51h, 52h, 52h, 53h, 54h, 54h, 55h, 56h, 56h, 57h, 58h
	dt		59h, 59h, 5ah, 5bh, 5bh, 5ch, 5dh, 5eh, 5eh, 5fh, 60h, 60h, 61h, 62h, 62h, 63h
	dt		64h, 65h, 65h, 66h, 67h, 67h, 68h, 69h, 6ah, 6ah, 6bh, 6ch, 6ch, 6dh, 6eh, 6eh
	dt		6fh, 70h, 71h, 71h, 72h, 73h, 73h, 74h, 75h, 76h, 76h, 77h, 78h, 78h, 79h, 7ah
	dt		7ah, 7bh, 7ch, 7dh, 7dh, 7eh, 7fh, 7fh, 80h, 81h, 82h, 82h, 83h, 84h, 84h, 85h
	dt		86h, 86h, 87h, 88h, 89h, 89h, 8ah, 8bh, 8bh, 8ch, 8dh, 8eh, 8eh, 8fh, 90h, 90h
	dt		91h, 92h, 92h, 93h, 94h, 95h, 95h, 96h, 97h, 97h, 98h, 99h, 9ah, 9ah, 9bh, 9ch
	dt		9ch, 9dh, 9eh, 9eh, 9fh, 0a0h, 0a1h, 0a1h, 0a2h, 0a3h, 0a3h, 0a4h, 0a5h, 0a6h, 0a6h, 0a7h
	dt		0a8h, 0a8h, 0a9h, 0aah, 0aah, 0abh, 0ach, 0adh, 0adh, 0aeh, 0afh, 0afh, 0b0h, 0b1h, 0b2h, 0b2h
	dt		0b3h, 0b4h, 0b4h, 0b5h, 0b6h, 0b6h, 0b7h, 0b8h, 0b9h, 0b9h, 0bah, 0bbh, 0bbh, 0bch, 0bdh, 0beh
;------------------------------------------------------------------------------------------------------


	END


		 

